import world.*;
import image.*;

/**
 * To represent a city in the USA
 * @author Viera K. Proulx
 * @since 20 March 2011
 */
class City{
    String zip;
    String name;
    String state;
    Loc loc;
  
    /**
     * The standard full constructor
     * @param zip the given zip code
     * @param name the name of this city
     * @param state the state for this city
     * @param loc the location data
     */
    City(String zip, String name, String state, Loc loc){
        this.zip = zip;
        this.name = name;
        this.state = state;
        this.loc = loc;
    }
  
    /**
     * The constructor with latitude and longitude given directly
     * @param zip the given zip code
     * @param name the name of this city
     * @param state the state for this city
     * @param longitude the longitude for this city
     * @param latitude the latitude for this city
     */  
    City(String zip, String name, String state, 
         int longitude, int latitude){
        this.zip = zip;
        this.name = name;
        this.state = state;
        this.loc = new Loc(longitude, latitude);
    }
  
    /* TEMPLATE:
       FIELDS
       ... this.name ...       -- String
       ... this.state ...      -- String
       ... this.loc ...        -- Loc
       ... this.zip ...        -- String
 
       METHODS
       ... this.distanceTo(City) ...       -- double
       ... this.drawCity(Scene) ...        -- Scene
 
       METHODS FOR FIELDS
       ... this.loc.distanceTo(Loc) ...       -- double
       ... this.loc.toPosn() ...              -- Posn
    */  
  
    /**
     * Compute the distance from this city to the given city
     * 
     * @param that the given city
     */
    double distanceTo(City that){
        return this.loc.distanceTo(that.loc);
    }
  
    /**
     * Produce the <code>Posn</code> for the location of this city 
     * on a 400 x 400 Scene
     * @return the desired location
     */
    Posn toPosn(){
        return this.loc.toPosn();
    }
  
    /**
     * Draw this city on the given <code>Scene<code> 
     * (assumed to be 400 x 400)
     * @param s the given <code>Scene</code>
     * @return the scene with this city shown
     */
    Scene drawCity(Scene s){
        return s.placeImage(new Circle(3, "solid", "red"), this.toPosn());
    }
  
    /**
     * Represent the information about this city
     * as a <code>String</code>
     * @return the <code>String</code> with all needed information
     * about this city
     */
    public String toString(){
        return this.name + ", " + this.state + ", " + this.zip +
            " (" + this.loc.toString() + ")"; 
    }
}
