import java.util.*;

/**
 * A class that represents the a graph representation
 * of the states in the central Europe
 * @author Viera K. Proulx
 *@since 30 March 2011
 */
public class EuroGraph extends Graph<EuroState>{
  
    /** a list of the names of the states that are included */
    ArrayList<EuroState> nodes = new ArrayList<EuroState>();
  
    /**
     * Constructor initializes the graph from the list of states
     */
    EuroGraph(){
        super();
        makeStates();
        initMap(this.nodes.iterator());
    }
  
    /**
     * Make a list of the 11 central European states
     * @return an <code>ArrayList</code> the 11 European states
     */
    public void makeStates(){
        EuroState swiss = new EuroState("Switzerland");
        EuroState dutch = new EuroState("Holland");
        EuroState belgium = new EuroState("Belgium");
        EuroState italy = new EuroState("Italy");
        EuroState france = new EuroState("France");
        EuroState germany = new EuroState("Germany");
        EuroState austria = new EuroState("Austria");
        EuroState slovakia = new EuroState("Slovakia");
        EuroState hungary = new EuroState("Hungary");
        EuroState czechs = new EuroState("Czechs");
        EuroState poland = new EuroState("Poland");
    
        swiss.addNeighbor("Austria", 300.0);
        swiss.addNeighbor("Italy", 350.0);
        swiss.addNeighbor("Germany", 550.0);
        swiss.addNeighbor("France", 250.0);
    
        dutch.addNeighbor("Belgium", 100.0);
        dutch.addNeighbor("Germany", 350.0);
    
        belgium.addNeighbor("Holland", 100.0);
        belgium.addNeighbor("France", 250.0);
        belgium.addNeighbor("Germany", 300.0);
       
        italy.addNeighbor("Switzerland", 350.0);
        italy.addNeighbor("Austria", 700.0); 
        italy.addNeighbor("France", 900.0); 
    
        france.addNeighbor("Italy", 900.0);
        france.addNeighbor("Germany", 800.0);
        france.addNeighbor("Switzerland", 250.0);
        france.addNeighbor("Belgium", 250.0);
    
        germany.addNeighbor("Austria", 700.0);
        germany.addNeighbor("Czechs", 300.0);
        germany.addNeighbor("Poland", 400.0);
        germany.addNeighbor("France", 800.0);
        germany.addNeighbor("Switzerland", 250.0);
        germany.addNeighbor("Holland", 350.0);
        germany.addNeighbor("Belgium", 300.0);
    
        austria.addNeighbor("Italy", 700.0);
        austria.addNeighbor("Germany", 700.0);
        austria.addNeighbor("Slovakia", 60.0);
        austria.addNeighbor("Czechs", 300.0);
        austria.addNeighbor("Hungary", 200.0);
        austria.addNeighbor("Switzerland", 300.0);
    
        slovakia.addNeighbor("Poland", 500.0);
        slovakia.addNeighbor("Austria", 60.0);
        slovakia.addNeighbor("Czechs", 360.0);
        slovakia.addNeighbor("Hungary", 150.0);
    
        hungary.addNeighbor("Austria", 200.0);
        hungary.addNeighbor("Slovakia", 150.0);
    
        poland.addNeighbor("Germany", 400.0);
        poland.addNeighbor("Slovakia", 500.0);
        poland.addNeighbor("Czechs", 600.0);
    
        czechs.addNeighbor("Austria", 300.0);
        czechs.addNeighbor("Germany", 300.0);
        czechs.addNeighbor("Slovakia", 360.0);
        czechs.addNeighbor("Poland", 600.0);
    
        this.nodes.add(austria);
        this.nodes.add(belgium);
        this.nodes.add(czechs);  
        this.nodes.add(dutch); 
        this.nodes.add(france);
        this.nodes.add(germany);
        this.nodes.add(hungary);
        this.nodes.add(italy);
        this.nodes.add(poland); 
        this.nodes.add(slovakia);
        this.nodes.add(swiss); 
    }
}
  
/**
 * To represent a European state with its neighbors
 * @author Viera K. Proulx
 * @since 30 March 2011
 */
class EuroState implements Node{
    protected String name;
    protected HashMap<String, Double> nearest = new HashMap<String, Double>();

    /** Construct this state with just the given name
     * (add neighbors later).
     * @param name the name of this state
     */
    protected EuroState(String name){
        this.name = name;
    }

    /**
     * produce the label for the node that represents this state
     * @return the name of this state
     */
    public String getLabel(){
        return this.name;
    }

    /**
     * produce a list of all neighbors of this state
     * @return the names of the neighboring states
     */
    public ArrayList<String> neighbors(){
        return new ArrayList<String>(this.nearest.keySet());
    }

    /**
     * produce the distance from this state to the given one
     * @param that the given state
     * @return the distance between these two states
     */
    public double distanceTo(Node that){
        return this.nearest.get(that.getLabel());
    }
  
    /**
     * Add the given neighbor to the list of neighbors for 
     * this state
     * @param name the name of the neighboring state to add
     * @param distance the distance between these two states
     */
    public void addNeighbor(String name, double distance){
        this.nearest.put(name, distance);
    }
}