import java.util.*;

/** To represent an arbitrary graph with nodes and distances
 * @author Viera K. Proulx
 * @since 20 March 2011
 */
public class Graph<T extends Node>{
    public HashMap<String, T> data;
  
    /**
     * Default constructor to allow user to extend the class
     */
    public Graph(){
        this.data = new HashMap<String, T>();
    }
  
    /**
     * Initialize this graph from the given list of nodes
     * @param nodes the given list of nodes
     */
    public Graph(Iterator<T> nodes){
        this.data = new HashMap<String, T>();
        this.initMap(nodes);
    }
  
    /**
     * Build the graph from the given list of nodes
     * @return an <code>HashMap</code> of the node labels and 
     * the information for each node
     */  
    public void initMap(Iterator<T> nodes){
        while (nodes.hasNext()){
            T node = nodes.next();
            this.data.put(node.getLabel(), node);
        }
    }
  
    /**
     * Produce a list of neighbors of the given node
     * @param label the label for the given node
     * @return the neighbors of the given node
     */
    public ArrayList<String> getNeighbors(String label){
        return this.data.get(label).neighbors();
    }
 
    /**
     * Compute the distance from the node represented by the
     * given <code>String</code> to another node represented
     * by the given <code>String</code>
     * @param goingFrom the origin node
     * @param goingTo the destination node
     * @return the distance between the two nodes
     */
    public double computeDistance(String goingFrom, String goingTo){
        Node node1 = this.data.get(goingFrom);
        Node node2 = this.data.get(goingTo);
        if (node1.neighbors().contains(goingTo))
            return node1.distanceTo(node2);
        else
            throw new IllegalArgumentException("Nodes " + node1.getLabel() + " and " + node2.getLabel() +
                                               " are not neighbors.");
    }
  
    /** 
     * Produce a <code>String</code> that represents this graph
     * @return the desired string
     */
    public String toString(){
        Collection<T> data = this.data.values();
        String s = "";
        for (T node : data){
            s = s + node.toString() + "\n";
        }
        return s;
    }
}
