import java.util.*;

/**
 * To represent a state in the US map
 * @author Viera K. Proulx
 * @since 20 March 2011
 */
public class State implements Node{
    String name;
    City city;
    ArrayList<String> neighbors;
  
    /**
     * The standard full constructor
     * @param name the name of this state
     * @param city the capitol city of this state
     * @param neighbors the list of neighboring states
     */
    public State(String name, City city, ArrayList<String> neighbors){
        this.name = name;
        this.city = city;
        this.neighbors = neighbors;
    }
  
    /** produce the label for this node
     * @return the name of this node */
    public String getLabel(){
        return this.name;
    }
  
    /** produce the list of labels for the neighbors of this node
     * @return the names of the neighboring nodes */
    public ArrayList<String> neighbors(){
        return this.neighbors;
    }
  
    /** compute the distance from this node to the given one
     * @param that the given node
     * @return the distance between these two nodes */
    public double distanceTo(Node node){
        State that = (State)node;
        return this.city.distanceTo(that.city);
    }
  
    /** Produce a <code>String</code> that represents this state
     * @return the desired string */
    public String toString(){
        return this.city.toString() + "\n" +
            "neighbors: " + toStringNeighbors();
    }
  
    /** Produce a <code>String</code> that represents 
     *  this neighbors of this state
     * @return the desired string */
    protected String toStringNeighbors(){
        String s = "";
        for (String n : this.neighbors){
            s = s + n + " ";
        }
        return s;
    }
}