/**
 * The interface that represents the collection of edges
 * that should be visited next in the graph traversal algorithms
 * @author Viera K. Proulx
 * @since 29 March 2011
 */
public interface ToDo{
    /** Add a new edge to the ToDo checklist
     * @param edge the edge we should add
     * @param path the path that has been already traveled
     */
    public void add(FromTo edge, Path path);

    /** Remove an edge from the ToDo checklist
     *  throw an exception if the checklist is empty
     * @return edge leading to the next node to be visited
     */
    public FromTo remove();
  
    /** Is this ToDo list empty?
     * @return true if there are no more nodes to visit
     */
    public boolean isEmpty();
  
    /** Does this ToDo list contain a link to the node with the given label?
     *  @param label the given label
     *  @return true if the node with the given given label 
     *   is a destination in the 'to do' list
     */
    public boolean contains(String label);
}
